import 'package:http/http.dart' as http;

import 'dart:async';
import 'dart:convert';

/// Returns list of definitions for specified word
Future<List<Definition>> getDefinitions(String word) async {
  var uri = Uri.https("api.urbandictionary.com", "v0/define", {"term": word});

  var res = await http.get(uri);
  var body = jsonDecode(utf8.decode(res.bodyBytes));

  List<Definition> buf = new List();

  for (var item in body['list'] as List<dynamic>)
    buf.add(Definition._new(item as Map<String, dynamic>));

  return buf;
}

/// Represents word definition fetched from api.
/// Individual fields can be null depending on amount of data returned by api
class Definition {
  /// Long definition of word.
  String definition;

  /// Url to UrbanDictionary webpage with this definition
  String permalink;

  /// Number of votes up on this definition
  int thumbsUp;

  /// Number of votes down on this definition
  int thumbsDown;

  /// List of urls of sounds for this definition
  List<String> soundUrls;

  /// Author of definition
  String author;

  /// Definitions word
  String word;

  /// Definition id
  int defId;

  /// Date when definition was created;
  DateTime writtenOn;

  /// Example of word usage
  String example;

  Definition._new(Map<String, dynamic> raw) {
    this.definition = raw['definition'] as String;
    this.permalink = raw['permalink'] as String;
    this.thumbsUp = raw['thumbs_up'] as int;
    this.soundUrls = raw['sound_urls'].cast<String>();
    this.author = raw['author'] as String;
    this.word = raw['word'] as String;
    this.defId = raw['defid'] as int;
    this.writtenOn = DateTime.parse(raw['written_on'] as String);
    this.example = raw['example'] as String;
    this.thumbsDown = raw['thumbs_down'] as int;
  }
}
